@@ -1,3 +1,5 @@
+from collections import defaultdict
+
 from libcoveofds.schema import OFDSSchema
 
 
@@ -8,37 +10,37 @@ class AdditionalCheckForNetwork:
         self._additional_check_results: list = []
         self._schema_object: OFDSSchema = schema_object
 
-    def check_node_first_pass(self, node: dict):
+    def check_node_first_pass(self, node: dict, path: str):
         pass
 
-    def check_span_first_pass(self, span: dict):
+    def check_span_first_pass(self, span: dict, path: str):
         pass
 
-    def check_phase_first_pass(self, phase: dict):
+    def check_phase_first_pass(self, phase: dict, path: str):
         pass
 
-    def check_organisation_first_pass(self, organisation: dict):
+    def check_organisation_first_pass(self, organisation: dict, path: str):
         pass
 
-    def check_contract_first_pass(self, contract: dict):
+    def check_contract_first_pass(self, contract: dict, path: str):
         pass
 
-    def check_node_second_pass(self, node: dict):
+    def check_node_second_pass(self, node: dict, path: str):
         pass
 
-    def check_span_second_pass(self, span: dict):
+    def check_span_second_pass(self, span: dict, path: str):
         pass
 
-    def check_phase_second_pass(self, phase: dict):
+    def check_phase_second_pass(self, phase: dict, path: str):
         pass
 
-    def check_organisation_second_pass(self, organisation: dict):
+    def check_organisation_second_pass(self, organisation: dict, path: str):
         pass
 
-    def check_contract_second_pass(self, contract: dict):
+    def check_contract_second_pass(self, contract: dict, path: str):
         pass
 
-    def get_additional_check_results(self):
+    def get_additional_check_results(self) -> list:
         return self._additional_check_results
 
     def skip_if_any_links_have_external_node_data(self) -> bool:
@@ -53,24 +55,32 @@ class SpansMustHaveValidNodesAdditionalCheckForNetwork(AdditionalCheckForNetwork
         super().__init__(schema_object)
         self._node_ids_seen: list = []
 
-    def check_node_first_pass(self, node: dict):
+    def check_node_first_pass(self, node: dict, path: str):
         id = node.get("id")
         if id:
             self._node_ids_seen.append(id)
 
-    def check_span_second_pass(self, span: dict):
+    def check_span_second_pass(self, span: dict, path: str):
         span_id = span.get("id")
         start = span.get("start")
-        if start:
-            self._check_node_id_valid(start, "span_start_node_not_found", span_id)
-        end = span.get("end")
-        if end:
-            self._check_node_id_valid(end, "span_end_node_not_found", span_id)
-
-    def _check_node_id_valid(self, node_id, error_type, span_id):
-        if not node_id in self._node_ids_seen:
+        if start and isinstance(start, str) and not start in self._node_ids_seen:
             self._additional_check_results.append(
-                {"type": error_type, "missing_node_id": node_id, "span_id": span_id}
+                {
+                    "type": "span_start_node_not_found",
+                    "missing_node_id": start,
+                    "span_id": span_id,
+                    "path": path + "/start",
+                }
+            )
+        end = span.get("end")
+        if end and isinstance(end, str) and not end in self._node_ids_seen:
+            self._additional_check_results.append(
+                {
+                    "type": "span_end_node_not_found",
+                    "missing_node_id": end,
+                    "span_id": span_id,
+                    "path": path + "/end",
+                }
             )
 
     def skip_if_any_links_have_external_node_data(self) -> bool:
@@ -81,9 +91,9 @@ class SpansMustHaveValidNodesAdditionalCheckForNetwork(AdditionalCheckForNetwork
 
 
 class NodesLocationAndSpansRouteAdditionalCheckForNetwork(AdditionalCheckForNetwork):
-    def check_node_first_pass(self, node: dict):
+    def check_node_first_pass(self, node: dict, path: str):
         location = node.get("location")
-        if location:
+        if isinstance(location, dict) and location:
             type = location.get("type")
             if type != "Point":
                 self._additional_check_results.append(
@@ -91,6 +101,7 @@ class NodesLocationAndSpansRouteAdditionalCheckForNetwork(AdditionalCheckForNetw
                         "type": "node_location_type_incorrect",
                         "node_id": node.get("id"),
                         "incorrect_type": type,
+                        "path": path + "/location/type",
                     }
                 )
             if not self._is_json_coordinates(location.get("coordinates")):
@@ -99,12 +110,13 @@ class NodesLocationAndSpansRouteAdditionalCheckForNetwork(AdditionalCheckForNetw
                         "type": "node_location_coordinates_incorrect",
                         "node_id": node.get("id"),
                         "incorrect_coordinates": location.get("coordinates"),
+                        "path": path + "/location/coordinates",
                     }
                 )
 
-    def check_span_first_pass(self, span: dict):
+    def check_span_first_pass(self, span: dict, path: str):
         location = span.get("route")
-        if location:
+        if isinstance(location, dict) and location:
             type = location.get("type")
             if type != "LineString":
                 self._additional_check_results.append(
@@ -112,6 +124,7 @@ class NodesLocationAndSpansRouteAdditionalCheckForNetwork(AdditionalCheckForNetw
                         "type": "span_route_type_incorrect",
                         "span_id": span.get("id"),
                         "incorrect_type": type,
+                        "path": path + "/route/type",
                     }
                 )
             if not self._is_json_list_coordinates(location.get("coordinates")):
@@ -120,6 +133,7 @@ class NodesLocationAndSpansRouteAdditionalCheckForNetwork(AdditionalCheckForNetw
                         "type": "span_route_coordinates_incorrect",
                         "span_id": span.get("id"),
                         "incorrect_coordinates": location.get("coordinates"),
+                        "path": path + "/route/coordinates",
                     }
                 )
 
@@ -145,65 +159,85 @@ class PhaseReferenceAdditionalCheckForNetwork(AdditionalCheckForNetwork):
         super().__init__(schema_object)
         self._phases: dict = {}
 
-    def check_phase_first_pass(self, phase: dict):
+    def check_phase_first_pass(self, phase: dict, path: str):
         id = phase.get("id")
         name = phase.get("name")
         if id:
             self._phases[id] = name
 
-    def check_node_second_pass(self, node: dict):
+    def check_node_second_pass(self, node: dict, path: str):
         if "phase" in node and isinstance(node["phase"], dict):
             self._check_related_phase_object(
                 node["phase"],
                 {
                     "type": "node_phase_reference_id_not_found",
                     "node_id": node.get("id"),
+                    "path": path + "/phase/id",
                 },
                 {
                     "type": "node_phase_reference_name_does_not_match",
                     "node_id": node.get("id"),
+                    "path": path + "/phase/name",
                 },
                 {
                     "type": "node_phase_reference_name_set_but_not_in_original",
                     "node_id": node.get("id"),
+                    "path": path + "/phase/name",
                 },
             )
 
-    def check_span_second_pass(self, span: dict):
+    def check_span_second_pass(self, span: dict, path: str):
         if "phase" in span and isinstance(span["phase"], dict):
             self._check_related_phase_object(
                 span["phase"],
                 {
                     "type": "span_phase_reference_id_not_found",
                     "span_id": span.get("id"),
+                    "path": path + "/phase/id",
                 },
                 {
                     "type": "span_phase_reference_name_does_not_match",
                     "span_id": span.get("id"),
+                    "path": path + "/phase/name",
                 },
                 {
                     "type": "span_phase_reference_name_set_but_not_in_original",
                     "span_id": span.get("id"),
+                    "path": path + "/phase/name",
                 },
             )
 
-    def check_contract_second_pass(self, contract: dict):
+    def check_contract_second_pass(self, contract: dict, path: str):
         if "relatedPhases" in contract and isinstance(contract["relatedPhases"], list):
-            for related_phase in contract["relatedPhases"]:
+            for related_phase_idx, related_phase in enumerate(
+                contract["relatedPhases"]
+            ):
                 if isinstance(related_phase, dict):
                     self._check_related_phase_object(
                         related_phase,
                         {
                             "type": "contract_related_phase_reference_id_not_found",
                             "contract_id": contract.get("id"),
+                            "path": path
+                            + "/relatedPhases/"
+                            + str(related_phase_idx)
+                            + "/id",
                         },
                         {
                             "type": "contract_related_phase_reference_name_does_not_match",
                             "contract_id": contract.get("id"),
+                            "path": path
+                            + "/relatedPhases/"
+                            + str(related_phase_idx)
+                            + "/name",
                         },
                         {
                             "type": "contract_related_phase_reference_name_set_but_not_in_original",
                             "contract_id": contract.get("id"),
+                            "path": path
+                            + "/relatedPhases/"
+                            + str(related_phase_idx)
+                            + "/name",
                         },
                     )
 
@@ -230,7 +264,9 @@ class PhaseReferenceAdditionalCheckForNetwork(AdditionalCheckForNetwork):
                     )
                 # check - if names are both set, do they match?
                 if name and self._phases[id] and name != self._phases[id]:
-                    name_not_match_result.update({"name_in_reference": name})
+                    name_not_match_result.update(
+                        {"name_in_reference": name, "name_should_be": self._phases[id]}
+                    )
                     self._additional_check_results.append(name_not_match_result)
             else:
                 # check failed - id is not known
@@ -243,13 +279,13 @@ class OrganisationReferenceAdditionalCheckForNetwork(AdditionalCheckForNetwork):
         super().__init__(schema_object)
         self._organisations: dict = {}
 
-    def check_organisation_first_pass(self, organisation: dict):
+    def check_organisation_first_pass(self, organisation: dict, path: str):
         id = organisation.get("id")
         name = organisation.get("name")
         if id:
             self._organisations[id] = name
 
-    def check_node_second_pass(self, node: dict):
+    def check_node_second_pass(self, node: dict, path: str):
         if "physicalInfrastructureProvider" in node and isinstance(
             node["physicalInfrastructureProvider"], dict
         ):
@@ -259,39 +295,58 @@ class OrganisationReferenceAdditionalCheckForNetwork(AdditionalCheckForNetwork):
                     "type": "node_organisation_reference_id_not_found",
                     "node_id": node.get("id"),
                     "field": "physicalInfrastructureProvider",
+                    "path": path + "/physicalInfrastructureProvider/id",
                 },
                 {
                     "type": "node_organisation_reference_name_does_not_match",
                     "node_id": node.get("id"),
                     "field": "physicalInfrastructureProvider",
+                    "path": path + "/physicalInfrastructureProvider/name",
                 },
                 {
                     "type": "node_organisation_reference_name_set_but_not_in_original",
                     "node_id": node.get("id"),
                     "field": "physicalInfrastructureProvider",
+                    "path": path + "/physicalInfrastructureProvider/name",
                 },
             )
-        if "networkProvider" in node and isinstance(node["networkProvider"], dict):
-            self._check_related_organisation_object(
-                node["networkProvider"],
-                {
-                    "type": "node_organisation_reference_id_not_found",
-                    "node_id": node.get("id"),
-                    "field": "networkProvider",
-                },
-                {
-                    "type": "node_organisation_reference_name_does_not_match",
-                    "node_id": node.get("id"),
-                    "field": "networkProvider",
-                },
-                {
-                    "type": "node_organisation_reference_name_set_but_not_in_original",
-                    "node_id": node.get("id"),
-                    "field": "networkProvider",
-                },
-            )
+        if "networkProviders" in node and isinstance(node["networkProviders"], list):
+            for network_provider_idx, network_provider in enumerate(
+                node["networkProviders"]
+            ):
+                if isinstance(network_provider, dict):
+                    self._check_related_organisation_object(
+                        network_provider,
+                        {
+                            "type": "node_organisation_reference_id_not_found",
+                            "node_id": node.get("id"),
+                            "field": "networkProviders",
+                            "path": path
+                            + "/networkProviders/"
+                            + str(network_provider_idx)
+                            + "/id",
+                        },
+                        {
+                            "type": "node_organisation_reference_name_does_not_match",
+                            "node_id": node.get("id"),
+                            "field": "networkProviders",
+                            "path": path
+                            + "/networkProviders/"
+                            + str(network_provider_idx)
+                            + "/name",
+                        },
+                        {
+                            "type": "node_organisation_reference_name_set_but_not_in_original",
+                            "node_id": node.get("id"),
+                            "field": "networkProviders",
+                            "path": path
+                            + "/networkProviders/"
+                            + str(network_provider_idx)
+                            + "/name",
+                        },
+                    )
 
-    def check_span_second_pass(self, span: dict):
+    def check_span_second_pass(self, span: dict, path: str):
         if "physicalInfrastructureProvider" in span and isinstance(
             span["physicalInfrastructureProvider"], dict
         ):
@@ -301,37 +356,56 @@ class OrganisationReferenceAdditionalCheckForNetwork(AdditionalCheckForNetwork):
                     "type": "span_organisation_reference_id_not_found",
                     "span_id": span.get("id"),
                     "field": "physicalInfrastructureProvider",
+                    "path": path + "/physicalInfrastructureProvider/id",
                 },
                 {
                     "type": "span_organisation_reference_name_does_not_match",
                     "span_id": span.get("id"),
                     "field": "physicalInfrastructureProvider",
+                    "path": path + "/physicalInfrastructureProvider/name",
                 },
                 {
                     "type": "span_organisation_reference_name_set_but_not_in_original",
                     "span_id": span.get("id"),
                     "field": "physicalInfrastructureProvider",
+                    "path": path + "/physicalInfrastructureProvider/name",
                 },
             )
-        if "networkProvider" in span and isinstance(span["networkProvider"], dict):
-            self._check_related_organisation_object(
-                span["networkProvider"],
-                {
-                    "type": "span_organisation_reference_id_not_found",
-                    "span_id": span.get("id"),
-                    "field": "networkProvider",
-                },
-                {
-                    "type": "span_organisation_reference_name_does_not_match",
-                    "span_id": span.get("id"),
-                    "field": "networkProvider",
-                },
-                {
-                    "type": "span_organisation_reference_name_set_but_not_in_original",
-                    "span_id": span.get("id"),
-                    "field": "networkProvider",
-                },
-            )
+        if "networkProviders" in span and isinstance(span["networkProviders"], list):
+            for network_provider_idx, network_provider in enumerate(
+                span["networkProviders"]
+            ):
+                if isinstance(network_provider, dict):
+                    self._check_related_organisation_object(
+                        network_provider,
+                        {
+                            "type": "span_organisation_reference_id_not_found",
+                            "span_id": span.get("id"),
+                            "field": "networkProviders",
+                            "path": path
+                            + "/networkProviders/"
+                            + str(network_provider_idx)
+                            + "/id",
+                        },
+                        {
+                            "type": "span_organisation_reference_name_does_not_match",
+                            "span_id": span.get("id"),
+                            "field": "networkProviders",
+                            "path": path
+                            + "/networkProviders/"
+                            + str(network_provider_idx)
+                            + "/name",
+                        },
+                        {
+                            "type": "span_organisation_reference_name_set_but_not_in_original",
+                            "span_id": span.get("id"),
+                            "field": "networkProviders",
+                            "path": path
+                            + "/networkProviders/"
+                            + str(network_provider_idx)
+                            + "/name",
+                        },
+                    )
         if "supplier" in span and isinstance(span["supplier"], dict):
             self._check_related_organisation_object(
                 span["supplier"],
@@ -339,36 +413,42 @@ class OrganisationReferenceAdditionalCheckForNetwork(AdditionalCheckForNetwork):
                     "type": "span_organisation_reference_id_not_found",
                     "span_id": span.get("id"),
                     "field": "supplier",
+                    "path": path + "/supplier/id",
                 },
                 {
                     "type": "span_organisation_reference_name_does_not_match",
                     "span_id": span.get("id"),
                     "field": "supplier",
+                    "path": path + "/supplier/name",
                 },
                 {
                     "type": "span_organisation_reference_name_set_but_not_in_original",
                     "span_id": span.get("id"),
                     "field": "supplier",
+                    "path": path + "/supplier/name",
                 },
             )
 
-    def check_phase_second_pass(self, phase: dict):
+    def check_phase_second_pass(self, phase: dict, path: str):
         if "funders" in phase and isinstance(phase["funders"], list):
-            for funder in phase["funders"]:
+            for funder_idx, funder in enumerate(phase["funders"]):
                 if isinstance(funder, dict):
                     self._check_related_organisation_object(
                         funder,
                         {
                             "type": "phase_organisation_reference_id_not_found",
                             "phase_id": phase.get("id"),
+                            "path": path + "/funders/" + str(funder_idx) + "/id",
                         },
                         {
                             "type": "phase_organisation_reference_name_does_not_match",
                             "phase_id": phase.get("id"),
+                            "path": path + "/funders/" + str(funder_idx) + "/name",
                         },
                         {
                             "type": "phase_organisation_reference_name_set_but_not_in_original",
                             "phase_id": phase.get("id"),
+                            "path": path + "/funders/" + str(funder_idx) + "/name",
                         },
                     )
 
@@ -395,7 +475,12 @@ class OrganisationReferenceAdditionalCheckForNetwork(AdditionalCheckForNetwork):
                     )
                 # check - if names are both set, do they match?
                 if name and self._organisations[id] and name != self._organisations[id]:
-                    name_not_match_result.update({"name_in_reference": name})
+                    name_not_match_result.update(
+                        {
+                            "name_in_reference": name,
+                            "name_should_be": self._organisations[id],
+                        }
+                    )
                     self._additional_check_results.append(name_not_match_result)
             else:
                 # check failed - id is not known
@@ -406,11 +491,13 @@ class OrganisationReferenceAdditionalCheckForNetwork(AdditionalCheckForNetwork):
 class NodeInternationalConnectionCountryAdditionalCheckForNetwork(
     AdditionalCheckForNetwork
 ):
-    def check_node_first_pass(self, node: dict):
+    def check_node_first_pass(self, node: dict, path: str):
         if "internationalConnections" in node and isinstance(
             node["internationalConnections"], list
         ):
-            for international_connection in node["internationalConnections"]:
+            for international_connection_idx, international_connection in enumerate(
+                node["internationalConnections"]
+            ):
                 if isinstance(
                     international_connection, dict
                 ) and not international_connection.get("country"):
@@ -418,6 +505,9 @@ class NodeInternationalConnectionCountryAdditionalCheckForNetwork(
                         {
                             "type": "node_international_connections_country_not_set",
                             "node_id": node.get("id"),
+                            "path": path
+                            + "/internationalConnections/"
+                            + str(international_connection_idx),
                         }
                     )
 
@@ -427,7 +517,7 @@ class IsNodeUsedInSpanAdditionalCheckForNetwork(AdditionalCheckForNetwork):
         super().__init__(schema_object)
         self._node_ids_used_in_spans: list = []
 
-    def check_span_first_pass(self, span: dict):
+    def check_span_first_pass(self, span: dict, path: str):
         start = span.get("start")
         if start and start not in self._node_ids_used_in_spans:
             self._node_ids_used_in_spans.append(start)
@@ -435,13 +525,14 @@ class IsNodeUsedInSpanAdditionalCheckForNetwork(AdditionalCheckForNetwork):
         if end and end not in self._node_ids_used_in_spans:
             self._node_ids_used_in_spans.append(end)
 
-    def check_node_second_pass(self, node: dict):
+    def check_node_second_pass(self, node: dict, path: str):
         id = node.get("id")
         if id and id not in self._node_ids_used_in_spans:
             self._additional_check_results.append(
                 {
                     "type": "node_not_used_in_any_spans",
                     "node_id": node.get("id"),
+                    "path": path,
                 }
             )
 
@@ -452,6 +543,101 @@ class IsNodeUsedInSpanAdditionalCheckForNetwork(AdditionalCheckForNetwork):
         return True
 
 
+class UniqueIDsAdditionalCheckForNetwork(AdditionalCheckForNetwork):
+    def __init__(self, schema_object: OFDSSchema):
+        super().__init__(schema_object)
+        self._node_ids_seen: defaultdict = defaultdict(list)
+        self._span_ids_seen: defaultdict = defaultdict(list)
+        self._phase_ids_seen: defaultdict = defaultdict(list)
+        self._organisation_ids_seen: defaultdict = defaultdict(list)
+        self._contract_ids_seen: defaultdict = defaultdict(list)
+
+    def check_node_first_pass(self, node: dict, path: str):
+        id = node.get("id")
+        if id and isinstance(id, str):
+            self._node_ids_seen[id].append(path)
+
+    def check_span_first_pass(self, span: dict, path: str):
+        id = span.get("id")
+        if id and isinstance(id, str):
+            self._span_ids_seen[id].append(path)
+
+    def check_phase_first_pass(self, phase: dict, path: str):
+        id = phase.get("id")
+        if id and isinstance(id, str):
+            self._phase_ids_seen[id].append(path)
+
+    def check_organisation_first_pass(self, organisation: dict, path: str):
+        id = organisation.get("id")
+        if id and isinstance(id, str):
+            self._organisation_ids_seen[id].append(path)
+
+    def check_contract_first_pass(self, contract: dict, path: str):
+        id = contract.get("id")
+        if id and isinstance(id, str):
+            self._contract_ids_seen[id].append(path)
+
+    def get_additional_check_results(self) -> list:
+        out: list = []
+        for id, paths in self._node_ids_seen.items():
+            if len(paths) > 1:
+                for path in paths:
+                    out.append(
+                        {
+                            "type": "duplicate_node_id",
+                            "node_id": id,
+                            "path": path + "/id",
+                        }
+                    )
+        for id, paths in self._span_ids_seen.items():
+            if len(paths) > 1:
+                for path in paths:
+                    out.append(
+                        {
+                            "type": "duplicate_span_id",
+                            "span_id": id,
+                            "path": path + "/id",
+                        }
+                    )
+        for id, paths in self._phase_ids_seen.items():
+            if len(paths) > 1:
+                for path in paths:
+                    out.append(
+                        {
+                            "type": "duplicate_phase_id",
+                            "phase_id": id,
+                            "path": path + "/id",
+                        }
+                    )
+        for id, paths in self._organisation_ids_seen.items():
+            if len(paths) > 1:
+                for path in paths:
+                    out.append(
+                        {
+                            "type": "duplicate_organisation_id",
+                            "organisation_id": id,
+                            "path": path + "/id",
+                        }
+                    )
+        for id, paths in self._contract_ids_seen.items():
+            if len(paths) > 1:
+                for path in paths:
+                    out.append(
+                        {
+                            "type": "duplicate_contract_id",
+                            "contract_id": id,
+                            "path": path + "/id",
+                        }
+                    )
+        return out
+
+    def skip_if_any_links_have_external_node_data(self) -> bool:
+        return False
+
+    def skip_if_any_links_have_external_span_data(self) -> bool:
+        return False
+
+
 ADDITIONAL_CHECK_CLASSES_FOR_NETWORK = [
     SpansMustHaveValidNodesAdditionalCheckForNetwork,
     NodesLocationAndSpansRouteAdditionalCheckForNetwork,
@@ -459,21 +645,25 @@ ADDITIONAL_CHECK_CLASSES_FOR_NETWORK = [
     OrganisationReferenceAdditionalCheckForNetwork,
     NodeInternationalConnectionCountryAdditionalCheckForNetwork,
     IsNodeUsedInSpanAdditionalCheckForNetwork,
+    UniqueIDsAdditionalCheckForNetwork,
 ]
 
 
 class PythonValidate:
+    """Validates data using additional checks custom written in Python"""
+
     def __init__(self, schema: OFDSSchema):
         self._schema = schema
 
     def validate(self, json_data: dict) -> list:
+        """Call with data. Results are returned."""
 
         additional_checks: list = []
 
         # For each Network
         networks = json_data.get("networks")
         if isinstance(networks, list):
-            for network in networks:
+            for network_idx, network in enumerate(networks):
                 if isinstance(network, dict):
                     additional_check_instances = [
                         x(self._schema) for x in ADDITIONAL_CHECK_CLASSES_FOR_NETWORK
@@ -496,6 +686,7 @@ class PythonValidate:
                             {
                                 "network_id": network.get("id"),
                                 "type": "has_links_with_external_node_data",
+                                "path": "/networks/" + str(network_idx),
                             }
                         )
                     links_with_external_spans = [
@@ -515,6 +706,7 @@ class PythonValidate:
                             {
                                 "network_id": network.get("id"),
                                 "type": "has_links_with_external_span_data",
+                                "path": "/networks/" + str(network_idx),
                             }
                         )
                     nodes = network.get("nodes", [])
@@ -531,35 +723,87 @@ class PythonValidate:
                     contracts = contracts if isinstance(contracts, list) else []
                     # First pass
                     for additional_check_instance in additional_check_instances:
-                        for node in nodes:
-                            additional_check_instance.check_node_first_pass(node)
-                        for span in spans:
-                            additional_check_instance.check_span_first_pass(span)
-                        for phase in phases:
-                            additional_check_instance.check_phase_first_pass(phase)
-                        for organisation in organisations:
-                            additional_check_instance.check_organisation_first_pass(
-                                organisation
+                        for node_idx, node in enumerate(nodes):
+                            additional_check_instance.check_node_first_pass(
+                                node,
+                                "/networks/"
+                                + str(network_idx)
+                                + "/nodes/"
+                                + str(node_idx),
                             )
-                        for contract in contracts:
+                        for span_idx, span in enumerate(spans):
+                            additional_check_instance.check_span_first_pass(
+                                span,
+                                "/networks/"
+                                + str(network_idx)
+                                + "/spans/"
+                                + str(span_idx),
+                            )
+                        for phase_idx, phase in enumerate(phases):
+                            additional_check_instance.check_phase_first_pass(
+                                phase,
+                                "/networks/"
+                                + str(network_idx)
+                                + "/phases/"
+                                + str(phase_idx),
+                            )
+                        for organisation_idx, organisation in enumerate(organisations):
+                            additional_check_instance.check_organisation_first_pass(
+                                organisation,
+                                "/networks/"
+                                + str(network_idx)
+                                + "/organisations/"
+                                + str(organisation_idx),
+                            )
+                        for contract_idx, contract in enumerate(contracts):
                             additional_check_instance.check_contract_first_pass(
-                                contract
+                                contract,
+                                "/networks/"
+                                + str(network_idx)
+                                + "/contracts/"
+                                + str(contract_idx),
                             )
                     # Second pass
                     for additional_check_instance in additional_check_instances:
-                        for node in nodes:
-                            additional_check_instance.check_node_second_pass(node)
-                        for span in spans:
-                            additional_check_instance.check_span_second_pass(span)
-                        for phase in phases:
-                            additional_check_instance.check_phase_second_pass(phase)
-                        for organisation in organisations:
-                            additional_check_instance.check_organisation_second_pass(
-                                organisation
+                        for node_idx, node in enumerate(nodes):
+                            additional_check_instance.check_node_second_pass(
+                                node,
+                                "/networks/"
+                                + str(network_idx)
+                                + "/nodes/"
+                                + str(node_idx),
                             )
-                        for contract in contracts:
+                        for span_idx, span in enumerate(spans):
+                            additional_check_instance.check_span_second_pass(
+                                span,
+                                "/networks/"
+                                + str(network_idx)
+                                + "/spans/"
+                                + str(span_idx),
+                            )
+                        for phase_idx, phase in enumerate(phases):
+                            additional_check_instance.check_phase_second_pass(
+                                phase,
+                                "/networks/"
+                                + str(network_idx)
+                                + "/phases/"
+                                + str(phase_idx),
+                            )
+                        for organisation_idx, organisation in enumerate(organisations):
+                            additional_check_instance.check_organisation_second_pass(
+                                organisation,
+                                "/networks/"
+                                + str(network_idx)
+                                + "/organisations/"
+                                + str(organisation_idx),
+                            )
+                        for contract_idx, contract in enumerate(contracts):
                             additional_check_instance.check_contract_second_pass(
-                                contract
+                                contract,
+                                "/networks/"
+                                + str(network_idx)
+                                + "/contracts/"
+                                + str(contract_idx),
                             )
                     # Results
                     for additional_check_instance in additional_check_instances:
